#
# $Id: Client.pm,v 1.1.2.1 2002/04/05 09:48:49 pbaltz Exp $
# Copyright 2002 Sun Microsystems, Inc.  All rights reserved.
#
# client API for the Sauce::Service::Daemon just allows you to talk to
# the daemon programmatically
#

package Sauce::Service::Client;

use lib qw(/usr/sausalito/perl);
use Socket;
use IO::Socket::UNIX;
use Sauce::Service::Daemon;

my $SOCKET = '/usr/sausalito/init_daemon.socket';

sub new
{
	my $self = shift;
	my $class = ref($self) || $self;
	$self = bless({}, $class);
	$self->init(@_);
	return $self;
}

sub init
{
	my ($self, @args) = @_;

	$self->{connected} = 0;
}

sub connect
{
	my ($self, @args) = @_;

	my $sock = new IO::Socket::UNIX('Type' => SOCK_STREAM,
					'Peer' => $SOCKET);
	if (!$sock) {
		# can't connect, try to start up the daemon
		my $pid = fork();
		if (!defined($pid)) {
			die("Can't fork: $!\n");
		} elsif ($pid == 0) {
			$SIG{CHLD} = 'DEFAULT';
			my $daemon = new Sauce::Service::Daemon();
			# this never returns
			$daemon->run();
		} else {
			waitpid($pid, 0);
		}

		# try to connect again
		$sock = new IO::Socket::UNIX('Type' => SOCK_STREAM,
					     'Peer' => $SOCKET);
		if (!$sock) {
			# nothing else we can do
			return(0);
		}
	}

	$self->{connected} = 1;
	$self->{rdsock} = $sock;
	$self->{wrsock} = $sock;

	return(1);
}

sub register_event
{
	my ($self, $service, $event) = @_;
	
	if ($service eq '' || $event eq '') {
		# what the heck am I supposed to do with that
		return(0);
	}

	my $wrsock = $self->{wrsock};
	my $rdsock = $self->{rdsock};

	print $wrsock "$service $event\n";

	my $response = <$rdsock>;
	if ($response =~ /OK$/) {
		return(1);
	}

	return(0);
}

# bye is only here for future expansion
sub bye
{
	my ($self, @args) = @_;

	close($self->{rdsock});

	# nothing important to do, so just return true
	return(1);
}

1;
